// this file contains the /turn_control and /turn_side datums.  /turn_control keeps two variables (side_one and side_two) which are /turn_sides.
// /turn_control:
// 		it's parent variable is the /battle
// 		current_turn variable is whichever turn(side_one or side_two) it is
// 		keeps a list of turn_ids (prefixed by "T")
// 		keeps a list of all mob in the battle
//		controls and alternates each side's turn
// 		locates mobs where they need to go when the battle is created and when it ends
// /turn_side:
//		keeps a list of members of it's side
// 		keeps a list of the member's old locations
// 		controls and alternates each member's turn

turn_control
	var
		turn_side
			side_one
			side_two
		input_mode
		battle/parent
		turn = 0
		turn_side/current_turn
		list/turn_ids = list()
		list/all_members

	New(battle/parent, list/one, list/two, list/all)
		if(!one.len || !two.len) del src
		if(!parent) del src

		if(!all.len)
			all = one.Copy()
			for(var/v in two) all.Add(v)

		all_members = all.Copy()

		src.parent = parent
		input_mode = src.parent.input_mode
		var/id = generate_tag("T", turn_ids)
		side_one = new(id, src, one)
		turn_ids += id
		id = generate_tag("T", turn_ids)
		side_two = new(id, src, two)
		turn_ids += id

	proc
		battle()
			return parent.clients
		get_next()
			if(input_mode == ALTERNATE_SIDES)
				if(current_turn == side_one)
					return side_two.get_next()
				return side_one.get_next()
			else
				if(current_turn == side_one)
					return side_one.get_next()
				return side_two.get_next()

		next_turn()
			if(input_mode == ALTERNATE_SIDES)
				if(current_turn == side_one)
					current_turn = side_two
				else
					current_turn = side_one

				turn += 1
				current_turn.current_turn = current_turn.get_next()
				current_turn.begin()
			else
				if(isnull(current_turn.get_next()))
					return end_turn(current_turn)
				else
					current_turn.current_turn = current_turn.get_next()
					current_turn.begin()

		// I just found it more familiar to do all_members() << "bla" than all_members << "bla".  It doesn't really matter.
		all_members()
			return all_members

		// This begins the turns.
		begin()
			turn = 1
			side_one.current_turn = side_one.members[1]
			side_two.current_turn = side_two.members[1]
			current_turn = pick_side()
			current_turn.begin()

		pick_side()
			return side_one

		// This ends one turn_side's turn and starts the next (checks to see if the battle is over before doing so)
		end_turn(turn_side/t)

			// call every mob in the ending turn's members side_turn_ended() proc
			for(var/mob/m in t.members)
				m.side_turn_ended()

			// if the battle is over, end it
			if(!isnull(parent.is_over()))
				return parent.end(parent.is_over())

			turn += 1
			if(t == side_one)
				current_turn = side_two
			else
				current_turn = side_one
			current_turn.current_turn = current_turn.members[1]
			current_turn.begin()

		// this sends all the mobs to where they need to be
		// takes their old locs by default.  when a battle is created it overrides this, so to send them back you don't have to do much.
		locate_mobs(list/one = side_one.locs, list/two = side_two.locs, z, od, td)
			if(!one.len || !two.len) return

			var/mob/m
			var/list/actual_one = list()
			var/list/actual_two = list()
			for(var/i = 1 to one.len)
				if(side_one.members.len < i) break
				m = side_one.members[i]
				m.dir = od

				if(isobj(one[i]))
					var/battle_info/b = one[i]
					m.loc = locate(b.x, b.y, z)
					actual_one.Add(m)
				else if(ismob(one[i]))
					m.loc = one[one[i]]

			for(var/i = 1 to two.len)
				if(side_two.members.len < i) break
				m = side_two.members[i]
				m.dir = td

				if(isobj(two[i]))
					var/battle_info/b = two[i]
					m.loc = locate(b.x, b.y, z)
					actual_two.Add(m)
				else if(ismob(two[i]))
					m.loc = two[two[i]]
			side_one.members = actual_one.Copy()
			side_two.members = actual_two.Copy()



turn_side
	var
		turn_control/parent
		list
			members = list()
			locs = list()
		mob/current_turn

		tmp
			run = FALSE

	New(id, turn_control/turn_control, list/members)
		if(!id) del src
		if(!turn_control) del src
		if(!members.len) del src

		tag = id
		parent = turn_control
		src.members = members.Copy()
		for(var/mob/m in src.members)
			locs[m] = m.loc
			m.turn = src

	proc
		battle()
			return parent.battle()

		input_mode()
			return parent.input_mode

		get_next()

			// get the index and add 1 to it.
			var/p = members.Find(current_turn)

			if(members.len > p)
				return members[p + 1]

			// if we are at the end of the list...
			else
				// if we are alternating sides, start from the beginning
				if(parent.input_mode == ALTERNATE_SIDES)
					return members[1]

				// otherwise return null, which will end the turn
				else return null

		get_enemies()
			if(src == parent.side_one)
				return parent.side_two.members
			return parent.side_one.members

		// This is called at the beginning of each turn.  It calls turn_started()
		begin()
			turn_started(current_turn)

		// Can m act?  Return TRUE or FALSE.  Override this to add anymore checks, just copy these two lines into the beginning of your override.
		// IMPORTANT: Calling ..() at the beginning will cause it to never reach your code, since this section of code will ALWAYS return either TRUE or FALSE before ending.
		can_act(mob/m)
			. = FALSE
			// if the mob can act, this will set . to TRUE
			if(m == current_turn && src == parent.current_turn) . =  TRUE

		// This is called to start the next turn.
		next_turn(mob/m)
			if(!m) return
			if(!(m in members)) return

			parent.next_turn()

		turn_ended(mob/m)
			m.turn_ended()
			// if the battle is over, end it
			if(!isnull(parent.parent.is_over()))
				return parent.parent.end(parent.parent.is_over())

			next_turn(m)

		turn_started(mob/m)
			m.my_turn()
			// So m did something.  end their turn.
			turn_ended(m)